@@ -10,7 +10,7 @@ import tempfile
 
 app = typer.Typer(help="Package manager for text adventure games.", name="mait")
 
-HOST = 'temperatecafe.com'
+HOST = '54.159.187.169'
 PORT = 6440  # TCP is 8081.
 
 SERVER = (HOST, PORT)
@@ -29,12 +29,10 @@ print(r"""
 -------------------------------------------------------------     """)
 
 
-# TODO: Convert to UDP-like commands.
 
 @app.command(help="Lists all packages in mait's registry.")
 def list():
     with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
-        s.settimeout(1.0)  # Timeout
         s.sendto(b'0', SERVER)  # Send message
         try:
             response = s.recvfrom(1024)  # Response from server.
@@ -138,6 +136,55 @@ Description:
         s.close()
         execute(game)
 
+@app.command(help="Permanently stores game files.")
+def install(game, path : str=f''):
+    if path == '':
+        path = os.path.expanduser('~') + '/.mait'
+
+    does_exist = os.path.exists(path)
+    if does_exist == False:
+        os.makedirs(path)
+
+    with socket.socket(socket.AF_INET, socket.SOCK_DGRAM) as s:
+        s.settimeout(60)
+        with alive_bar(4) as bar:
+            command = bytes("1".encode('UTF-8') + game.encode('UTF-8'))
+            s.sendto(command, SERVER)
+
+            l = s.recvfrom(1024)
+            size = l[0]
+            size = len(size) * 2
+            bar()
+            print(size)
+            f = open(f'{path}/{game}.zip', 'wb')
+            l = s.recvfrom(size)
+            f.write(l[0])
+            f.close()
+            bar()
+            # https://stackoverflow.com/questions/3083235/unzipping-file-results-in-badzipfile-file-is-not-a-zip-file
+            # God bless their souls.
+            f = open(f'{path}/{game}.zip', 'r+b')
+            data = f.read()
+            pos = data.find(b'\x50\x4b\x05\x06')  # End of central directory signature
+            if (pos > 0):
+                f.seek(pos + 22)  # size of 'ZIP end of central directory record'
+                f.truncate()
+                f.close()
+            else:
+                return
+
+            bar()
+
+            shutil.unpack_archive(f'{path}/{game}.zip', f'{path}/{game}')
+            os.remove(f'{path}/{game}.zip')
+            bar()
+
+        print(f"Game is stored at: {path}/{game}")
+
+        try:
+            os.mkdir(f'{path}/{game}')
+        except FileExistsError:
+            s.close()
 
 @app.command(help="Uploads a package to mait's registry.")
 def upload(path: str):
